import org.garret.jipc.*;

class Account implements java.io.Serializable { 
    long amount;
    Account(long value) { 
	amount = value;
    }
}


/**
 * Test for shared and exclusive locks
 * Usage: start N processes by "java Bank N" command
 * Output: none
 */
public class Bank extends JIPCTest { 
    static final int    nIterations = 1000;

    public static void main(String args[]) throws Exception { 
	if (args.length < 1) { 
	    System.out.println("Usage: java Bank <number-of-processes>");
	    return;
	}
	int nProcesses = Integer.parseInt(args[0], 10);
	JIPCSession session = createSession();
	long initialAmount = nIterations*nProcesses;
	Account srcAcc = new Account(initialAmount);
	Account dstAcc = new Account(0);	
	JIPCSharedMemory srcShm = session.createSharedMemory("src", srcAcc);
	JIPCSharedMemory dstShm = session.createSharedMemory("dst", dstAcc);
	JIPCLock srcLock = session.createLock("src");
	JIPCLock dstLock = session.createLock("dst");
	JIPCBarrier barrier = session.createBarrier("bank", nProcesses);

	for (int i = 1; i <= nIterations; i++) { 
	    srcLock.sharedLock();
	    dstLock.sharedLock();
	    srcAcc = (Account)srcShm.get();
	    dstAcc = (Account)dstShm.get();
	    Assert.that(srcAcc.amount + dstAcc.amount == initialAmount);
	    srcLock.unlock();
	    dstLock.unlock();
	    srcLock.exclusiveLock();
	    dstLock.exclusiveLock();
	    srcAcc = (Account)srcShm.get();
	    dstAcc = (Account)dstShm.get();
	    srcAcc.amount -= 1;
	    dstAcc.amount += 1;
	    srcShm.set(srcAcc);
	    dstShm.set(dstAcc);
	    dstLock.unlock();
	    srcLock.unlock();
	}
	barrier.waitFor();
	srcAcc = (Account)srcShm.get();
	dstAcc = (Account)dstShm.get();
	Assert.that(srcAcc.amount == 0);
	Assert.that(dstAcc.amount == initialAmount);
	srcShm.close();
	dstShm.close();
	srcLock.close();
	dstLock.close();
	barrier.close();
	session.close();
    }
}




