import org.garret.jipc.*;
import java.util.Random;

/**
 * Test for deadlock detection
 * Usage:
 * Start N processes with "java Deadlock N" command
 * To disable deadlocks, specify "jipc.nodeadlock" system property
 * To enable lock with trimeouts, specify "jipc.timed" system property
 * Output: "Number of detected deadlocks: XXX"
 */
public class Deadlock extends JIPCTest { 
    static final int nIterations = 1000;
    static final long timeout = 100;

    public static void main(String args[]) throws Exception { 
	JIPCSession session = createSession();
	if (args.length < 1) { 
	    System.out.println("Usage: java Deadlock <number-of-participants>");
	    return;
	}
	Random rand = new Random(2002);
	int nParticipants = Integer.parseInt(args[0], 10);	
	JIPCBarrier barrier = session.createBarrier("barrier", nParticipants);
	JIPCLock lock[] = new JIPCLock[nParticipants];
	for (int i = 0; i < nParticipants; i++) { 
	    lock[i] = session.createLock("lock" + i);
	}
	boolean noDeadlocks = Boolean.getBoolean("jipc.nodeadlock");
	boolean timed = Boolean.getBoolean("jipc.timed");
	int nDeadlocks = 0;

	barrier.waitFor();
	for (int i = 0; i < nIterations; i++) { 
	    int firstLock = rand.nextInt(nParticipants);
	    int secondLock = rand.nextInt(nParticipants);
	    boolean firstLockIsExclusive = true;
	    boolean secondLockIsExclusive = true;
	    if (noDeadlocks) { 
		if (firstLock > secondLock) { 		
		    int tmp = firstLock;
		    firstLock = secondLock;
		    secondLock = tmp;
		}
	    } else { 
		firstLockIsExclusive = rand.nextBoolean();
		secondLockIsExclusive = rand.nextBoolean();
	    }
	    if (timed) { 
		boolean succeed;
		if (firstLockIsExclusive) { 
		    lock[firstLock].exclusiveLock();
		} else {
		    lock[firstLock].sharedLock();
		}
		if (secondLockIsExclusive) { 
		    succeed = lock[secondLock].exclusiveLock(timeout);
		} else {
		    succeed = lock[secondLock].sharedLock(timeout);
		}
		if (succeed) { 
		    if (firstLock != secondLock) { 
			lock[secondLock].unlock();
		    }
		} else { 
		    nDeadlocks += 1;
		}
	    } else { 
		if (firstLockIsExclusive) { 
		    lock[firstLock].exclusiveLock();
		} else {
		    lock[firstLock].sharedLock();
		}
		try { 
		    if (secondLockIsExclusive) { 
			lock[secondLock].exclusiveLock();
		    } else {
			lock[secondLock].sharedLock();
		    }
		    if (firstLock != secondLock) { 
			lock[secondLock].unlock();
		    }
		} catch (JIPCDeadlockException x) { 
		    nDeadlocks += 1;
		}
	    }
	    lock[firstLock].unlock();
	} 
	System.out.println("Number of detected deadlocks: " + nDeadlocks);
	session.close();
    }
}


