import org.garret.jipc.*;
import org.garret.jipc.server.*;
import org.garret.jipc.client.*;

/**
 * Base class common for all tests.
 * This class is responsible for establishing connection.
 * If "jipc.port" system property is specified then client session is created. JIPC server
 * should be started before by "java org.java.jipc.server.JIPCServer <port>" command
 * Host name is obtained from "jipc.host" system property, by default "localhost" is used.
 * If "jipc.port" system property is not defined or is empty string, then local server session is created.
 */
public class JIPCTest { 
    static JIPCSession createSession() throws Exception { 
	String host = System.getProperty("jipc.host", "localhost");
	String port = System.getProperty("jipc.port", "6000");
	JIPCSession session;

	if (port == null || port.length() == 0) { 
	    session = org.garret.jipc.server.JIPCServer.getInstance().create(host, 0);
	} else { 
	    int portNo = Integer.parseInt(port, 10);
	    session = org.garret.jipc.client.JIPCClientFactory.getInstance().create(host, portNo);
	} 
	Assert.that(session != null);
	return session;
    }
}
