import org.garret.jipc.*;
import java.util.Random;

/**
 * Test for mutex deadlock detection
 * Usage:
 * Start N processes with "java MutexDeadlock N" command
 * To disable deadlocks, specify "jipc.nodeadlock" system property
 * To enable lock with trimeouts, specify "jipc.timed" system property
 * Output: "Number of detected deadlocks: XXX"
 */
public class MutexDeadlock extends JIPCTest { 
    static final int nIterations = 1000;
    static final long timeout = 100;
  
    public static void main(String args[]) throws Exception { 
	JIPCSession session = createSession();
	if (args.length < 1) { 
	    System.out.println("Usage: java MutexDeadlock <number-of-participants>");
	    return;
	}
	Random rand = new Random(2002);
	int nParticipants = Integer.parseInt(args[0], 10);	
	JIPCBarrier barrier = session.createBarrier("barrier", nParticipants);
	JIPCMutex mutex[] = new JIPCMutex[nParticipants];
	for (int i = 0; i < nParticipants; i++) { 
	    mutex[i] = session.createMutex("mutex" + i, false);
	}
	boolean noDeadlocks = Boolean.getBoolean("jipc.nodeadlock");
	boolean timed = Boolean.getBoolean("jipc.timed");
	int nDeadlocks = 0;

	barrier.waitFor();
	for (int i = 0; i < nIterations; i++) { 
	    int firstLock = rand.nextInt(nParticipants);
	    int secondLock = rand.nextInt(nParticipants);
	    if (noDeadlocks && firstLock > secondLock) { 		
		int tmp = firstLock;
		firstLock = secondLock;
		secondLock = tmp;
	    }
	    if (timed) { 
		mutex[firstLock].lock(); 
		if (mutex[secondLock].lock(timeout)) { 
		    mutex[secondLock].unlock();
		} else { 
		    nDeadlocks += 1;
		}
	    } else {
		mutex[firstLock].lock();
		try { 
		    mutex[secondLock].lock();
		    mutex[secondLock].unlock();
		} catch (JIPCDeadlockException x) { 
		    nDeadlocks += 1;
		}
	    }
	    mutex[firstLock].unlock();
	} 
	System.out.println("Number of detected deadlocks: " + nDeadlocks);
	session.close();
    }
}





