import org.garret.jipc.*;
import java.util.Random;

/**
 * Philosophres at dinner.
 * Test for semaphores.
 * Usage: start N processes by "java Philosopher N" command
 * To disable deadlocks, specify "jipc.nodeadlock" system property
 * Output: "Number of faults: XXX"
 */
public class Philosophers extends JIPCTest { 
    static final int nIterations = 1000;
    static final int timeout = 100;
  
    public static void main(String args[]) throws Exception { 
	if (args.length < 1) { 
	    System.out.println("Usage: java Philosophers <number-of-philosophers>"); 
	    return;
	}
	int nPhilosophers = Integer.parseInt(args[0], 10);
	int nForks = nPhilosophers;
	JIPCSession session = createSession();
	JIPCSemaphore fork[] = new JIPCSemaphore[nForks];
	Random rand = new Random(2002);

	for (int i = 0; i < nForks; i++) { 
	    fork[i] = session.createSemaphore("fork" + i, 1);
	}
	boolean noDeadlocks = Boolean.getBoolean("jipc.nodeadlock");

	JIPCBarrier barrier = session.createBarrier("philosophers", nPhilosophers);
	barrier.waitFor();

	int nFaults = 0;
	for (int i = 0; i < nIterations; i++) { 
	    for (int j = 0; j < nPhilosophers; j++) { 
		int leftFork = j == 0 ? nForks-1 : j-1;
		int rightFork = j;
		int takeFirst, takeSecond;
		if (noDeadlocks) { 
		    if (leftFork > rightFork) { 
			takeFirst = rightFork;
			takeSecond = leftFork;
		    } else { 
			takeFirst = leftFork;
			takeSecond = rightFork;
		    }
		} else {
		    if (rand.nextBoolean()) { 
			takeFirst = leftFork;
			takeSecond = rightFork;
		    } else { 
			takeFirst = rightFork;
			takeSecond = leftFork;
		    }
		}
		fork[takeFirst].waitFor();
		if (!fork[takeSecond].waitFor(timeout)) { 
		    nFaults += 1;
		} else { 
		    fork[takeSecond].signal();
		}
		fork[takeFirst].signal();
	    }
	}
	System.out.println("Number of faults: " + nFaults);
	session.close();
    }
}
